/*
 * type for a field
 */
#define UNSIGNED 0
#define SIGNED 1
#define CHAR 2
#define DOUBLE 3
#define HEX 4
#define ADDRESS 5

#define PROFILE_VERSION 112
/*
 * version history
 *
 version 112: add zero-duration continuation intervals at beginning of frames
 version 111: 6-bit field->log_max_length instead of 7 bits
 version 110: add instruction addresses
 version 109: add arguments for all MPI routines in version 2.4
 version 108: add arguments for 12 specific MPI routines
              MPI_Send, MPI_Ssend, MPI_Rsend, MPI_Bsend, MPI_Recv
              MPI_Isend, MPI_Issend, MPI_Irsend, MPI_Ibsend, MPI_Irecv
              MPI_Sendrecv, MPI_Sendrecv_replace
 version 107: type field from 10 to 12 bits, node field from 8 to 12 bits
              cpu field from 4 to 6 bits, start/dura from double to 40 bits
 version 106: add ncpus for UTE_VERSION record
 version 105: add some arguments; now convert requires the binary profile
 version 104: field_format changed to allow type expansion
 version 103: add record names and marker names to the table
 version 102: TRCON event added for merging without global clock
 version 101: 8 bit thread ID with a thread table
 version 100: 32 bit thread ID
 */

/* 
 * current trace version from the UTE trace library
 */
#define TRACE_VERSION 53

/*
 * the log size of event tables
 */
#define LOG_EVENT_TYPES 12      /* 12 bits for event type */
#define NUM_BEBITS 2            /* two bebits */
#define LOG_TABLE_SIZE (LOG_EVENT_TYPES + NUM_BEBITS)       

#define LOG_BASIC_ETYPES 8      /* basic 256 MPI routines */
#define ETABLE_SIZE ((1 << (LOG_BASIC_ETYPES + NUM_BEBITS))+8)
#define MINBYTES 15             /* minimum number of bytes per record (107) */
#define ADDBYTES (MINBYTES+4)   /* minimum number of bytes with address */
#define NUM_ENTRIES (2 + 1 << LOG_BASIC_ETYPES)

/*
 * field_format
 */
typedef struct _field_format {
  unsigned vector:1;          /* a vector or a scalar field */
  unsigned vector_length:2;   /* number of bytes for vector counter; 0 for 
                                 4 bytes */
  unsigned type:6;            /* element type */
  unsigned log_max_length:6;  /* number of bits for the field */
  unsigned attribute:2;       /* number of bits for the field */
  unsigned name_index:15;     /* field name index */
} field_format;

/*
 * record_format
 */
typedef struct _record_format {
  unsigned short bitlength;     /* 1 byte if < 256, else 2 bytes */
  unsigned number_of_fields:8;  /* number of fields */
  unsigned number_of_sfields:8; /* number of scalar fields */
  unsigned short name_index;    /* record name index */
  field_format *field;          /* a list of field definitions */
} record_format;

/*
 * name buffer and name array for the table
 */
typedef struct _name_format {
  char *name_buffer;            /* a buffer */
  int buffer_size;              /* buffer size */
  int offset;                   /* current offset */
  char **name_array;            /* an array of pointers to buffer positions */
  int max_count;                /* array size */ 
  int count;                    /* current number of field names */
} name_format;

typedef struct _table_format {
  int version;                  /* version ID */
  name_format field_names;      /* field names */
  name_format record_names;     /* record names */
  record_format row[ETABLE_SIZE];
  name_format marker_names;     /* marker names */
  int log_table_size;           /* table size */
} table_format;

/*
 * global user marker name buffer size and max number of names
 */
#define MARKER_BUFF_SIZE 8192
#define MARKER_NNAMES 512


